@@ -1,6 +1,17 @@
 package Net::Shadowsocks;
 
-# ABSTRACT: An asynchronous, non-bloking shadowsocks client and server module.
+# ABSTRACT: An asynchronous, non-blocking shadowsocks client and server module.
+use 5.006;
+use strict;
+use warnings;
+use Crypt::Random qw( makerandom_octet );
+
+
+
+our $VERSION = '0.6.1';
+
+
+
 {
 
     package Net::Shadowsocks::Client;
@@ -20,20 +31,16 @@ package Net::Shadowsocks;
     use Mcrypt qw(:ALGORITHMS);
     use Mcrypt qw(:MODES);
     use Mcrypt qw(:FUNCS);
-    use Net::Shadowsocks qw(_EVP_BytesToKey);
+    use Net::Shadowsocks qw(_EVP_BytesToKey _initialize_cipher);
 
 
-
-    our $VERSION = '0.6.0';
-
-
-    sub new($$$$$)
+    sub new($$$$$$)
     {
 
         my ($class, %args) = @_;
         my $self = bless {
                           map { ($_ => $args{$_}) }
-                            qw(local_address local_port password server server_port),
+                            qw(local_address local_port password server server_port method),
                          }, $class;
         if ($self->{local_address} eq 'localhost')
         {
@@ -50,21 +57,12 @@ package Net::Shadowsocks;
 
                 #carp "Got new client connection: $client_host:$client_port";
                 my $addr_to_send = '';
-
-                my $iv = makerandom_octet(Length => 16);
                 my $serveriv;
-
-                #my $client;
                 my $stage = 0;
-
-                my $mode = 0;
-
-                my $encryptor =
-                  Mcrypt::mcrypt_load(Mcrypt::ARCFOUR, '', Mcrypt::STREAM, '');
-                my $decryptor =
-                  Mcrypt::mcrypt_load(Mcrypt::ARCFOUR, '', Mcrypt::STREAM, '');
-                my $key = Net::Shadowsocks::_EVP_BytesToKey(16, 16, '',
-                                                          $self->{password}, 1);
+                my $mode  = 0;
+                my ($encryptor, $decryptor, $key, $iv) =
+                  Net::Shadowsocks::_initialize_cipher($self->{method},
+                                                       $self->{password});
 
                 my $clienthandler;
                 my $remotehandler;
@@ -203,13 +201,24 @@ package Net::Shadowsocks;
                                                 $incomingdata =
                                                   substr($incomingdata,
                                                          length($iv));
-                                                my $md = Digest::MD5->new();
-                                                $md->add($key);
-                                                $md->add($serveriv);
-                                                my $decrypt_rc4_key =
-                                                  $md->digest();
-                                                Mcrypt::mcrypt_init($decryptor,
+                                                if ($self->{method} eq
+                                                    'rc4-md5')
+                                                {
+                                                    my $md = Digest::MD5->new();
+                                                    $md->add($key);
+                                                    $md->add($serveriv);
+                                                    my $decrypt_rc4_key =
+                                                      $md->digest();
+                                                    Mcrypt::mcrypt_init(
+                                                          $decryptor,
                                                           $decrypt_rc4_key, '');
+                                                }
+                                                else
+                                                {
+                                                    Mcrypt::mcrypt_init(
+                                                               $decryptor, $key,
+                                                               $serveriv);
+                                                }
                                             }
                                             my $decrypteddata =
                                               Mcrypt::mcrypt_decrypt($decryptor,
@@ -252,13 +261,20 @@ package Net::Shadowsocks;
                                 if ($addr_to_send ne '')
                                 {
                                     $plaindata = $addr_to_send . $client_buffer;
-                                    my $md = Digest::MD5->new();
-                                    $md->add($key);
-                                    $md->add($iv);
-                                    my $encrypt_rc4_key = $md->digest();
-                                    Mcrypt::mcrypt_init($encryptor,
-                                                        $encrypt_rc4_key, '');
-
+                                    if ($self->{method} eq 'rc4-md5')
+                                    {
+                                        my $md = Digest::MD5->new();
+                                        $md->add($key);
+                                        $md->add($iv);
+                                        my $encrypt_rc4_key = $md->digest();
+                                        Mcrypt::mcrypt_init($encryptor,
+                                                          $encrypt_rc4_key, '');
+                                    }
+                                    else
+                                    {
+                                        Mcrypt::mcrypt_init($encryptor, $key,
+                                                            $iv);
+                                    }
                                     $addr_to_send = '';
                                 }
                                 my $encrypteddata =
@@ -307,19 +323,17 @@ package Net::Shadowsocks;
     use Mcrypt qw(:ALGORITHMS);
     use Mcrypt qw(:MODES);
     use Mcrypt qw(:FUNCS);
-    use Net::Shadowsocks qw(_EVP_BytesToKey);
+    use Net::Shadowsocks qw(_EVP_BytesToKey _initialize_cipher);
 
 
-    our $VERSION = '0.6.0';
 
-
-    sub new($$$$$)
+    sub new($$$$$$)
     {
 
         my ($class, %args) = @_;
         my $self = bless {
                           map { ($_ => $args{$_}) }
-                            qw(local_address local_port password server server_port),
+                            qw(local_address local_port password server server_port method),
                          }, $class;
         if ($self->{server} eq 'localhost')
         {
@@ -335,22 +349,14 @@ package Net::Shadowsocks;
 
                 #carp "Got new client connection: $client_host:$client_port";
                 my $sent_iv = 0;
-
-                my $iv = makerandom_octet(Length => 16);
                 my $clientiv;
-
-                #my $client;
-                my $stage = 0;
-
                 my $mode = 0;
 
-                my $encryptor =
-                  Mcrypt::mcrypt_load(Mcrypt::ARCFOUR, '', Mcrypt::STREAM, '');
-                my $decryptor =
-                  Mcrypt::mcrypt_load(Mcrypt::ARCFOUR, '', Mcrypt::STREAM, '');
-                my $key = Net::Shadowsocks::_EVP_BytesToKey(16, 16, '',
-                                                          $self->{password}, 1);
-
+                #say $self->{method};
+                # say $self->{password};
+                my ($encryptor, $decryptor, $key, $iv) =
+                  Net::Shadowsocks::_initialize_cipher($self->{method},
+                                                       $self->{password});
                 my $clienthandler;
                 my $remotehandler;
                 $clienthandler = AnyEvent::Handle->new(
@@ -386,13 +392,20 @@ package Net::Shadowsocks;
                                   substr($client_buffer, 0, length($iv));
                                 $incomingdata =
                                   substr($client_buffer, length($iv));
-                                my $md = Digest::MD5->new();
-                                $md->add($key);
-                                $md->add($clientiv);
-                                my $decrypt_rc4_key = $md->digest();
-                                Mcrypt::mcrypt_init($decryptor,
-                                                    $decrypt_rc4_key, '');
-
+                                if ($self->{method} eq 'rc4-md5')
+                                {
+                                    my $md = Digest::MD5->new();
+                                    $md->add($key);
+                                    $md->add($clientiv);
+                                    my $decrypt_rc4_key = $md->digest();
+                                    Mcrypt::mcrypt_init($decryptor,
+                                                        $decrypt_rc4_key, '');
+                                }
+                                else
+                                {
+                                    Mcrypt::mcrypt_init($decryptor, $key,
+                                                        $clientiv);
+                                }
                                 $decrypteddata =
                                   Mcrypt::mcrypt_decrypt($decryptor,
                                                          $incomingdata);
@@ -418,7 +431,6 @@ package Net::Shadowsocks;
                                         $dest_port =
                                           unpack('n',
                                                  substr($decrypteddata, 5, 2));
-                                        say $dest_port;
                                         $decrypteddata =
                                           substr($decrypteddata, 7);
                                     }
@@ -503,13 +515,12 @@ package Net::Shadowsocks;
                                             carp "couldn't connect: $!";
                                             return;
                                         }
-                                        $stage = 4;
                                     },
                                     on_eof => sub {
                                         my $rhandler = shift;
-                                        $rhandler->destroy();
-                                        $mode = 0;
                                         AE::log info => "Remote: Done.";
+                                        $mode = 0;
+                                        $rhandler->destroy();
                                     },
                                     on_error => sub {
                                         my ($rhandler, $fatal, $msg) = @_;
@@ -523,12 +534,21 @@ package Net::Shadowsocks;
                                         my $plaindata = $remote_buffer;
                                         if ($sent_iv == 0)
                                         {
-                                            my $md = Digest::MD5->new();
-                                            $md->add($key);
-                                            $md->add($iv);
-                                            my $encrypt_rc4_key = $md->digest();
-                                            Mcrypt::mcrypt_init($encryptor,
+                                            if ($self->{method} eq 'rc4-md5')
+                                            {
+                                                my $md = Digest::MD5->new();
+                                                $md->add($key);
+                                                $md->add($iv);
+                                                my $encrypt_rc4_key =
+                                                  $md->digest();
+                                                Mcrypt::mcrypt_init($encryptor,
                                                           $encrypt_rc4_key, '');
+                                            }
+                                            else
+                                            {
+                                                Mcrypt::mcrypt_init($encryptor,
+                                                                    $key, $iv);
+                                            }
                                             $sent_iv = 1;
                                         }
                                         my $encrypteddata =
@@ -548,7 +568,6 @@ package Net::Shadowsocks;
                                         $remotehandler->{rbuf} = '';
                                     }
                                 );
-                                $stage = 5;
                             }
                             else
                             {
@@ -605,6 +624,36 @@ sub _EVP_BytesToKey($$$$$)
     return $key;
 }
 
+sub _initialize_cipher($$)
+{
+    my $method   = shift;
+    my $password = shift;
+
+    #say $method;
+    #say $password;
+    my $encryptor;
+    my $decryptor;
+    my $key;
+    my $iv;
+    if ($method eq 'rc4-md5')
+    {
+        $encryptor =
+          Mcrypt::mcrypt_load(Mcrypt::ARCFOUR, '', Mcrypt::STREAM, '');
+        $decryptor =
+          Mcrypt::mcrypt_load(Mcrypt::ARCFOUR, '', Mcrypt::STREAM, '');
+        $key = Net::Shadowsocks::_EVP_BytesToKey(16, 16, '', $password, 1);
+        $iv = makerandom_octet(Length => 16);
+    }
+    elsif ($method eq 'des-cfb')
+    {
+        $encryptor = Mcrypt::mcrypt_load(Mcrypt::DES, '', Mcrypt::CFB, '');
+        $decryptor = Mcrypt::mcrypt_load(Mcrypt::DES, '', Mcrypt::CFB, '');
+        $key = Net::Shadowsocks::_EVP_BytesToKey(8, 8, '', $password, 1);
+        $iv = makerandom_octet(Length => 8);
+    }
+    return $encryptor, $decryptor, $key, $iv;
+}
+
 
 1;    # End of Net::Shadowsocks
 
@@ -616,11 +665,11 @@ __END__
 
 =head1 NAME
 
-Net::Shadowsocks - An asynchronous, non-bloking shadowsocks client and server module.
+Net::Shadowsocks - An asynchronous, non-blocking shadowsocks client and server module.
 
 =head1 VERSION
 
-version 0.6.0
+version 0.6.1
 
 =head1 SYNOPSIS
 
@@ -639,20 +688,11 @@ supported is RC4-MD5 which is introduced into shadowsocks Python version 2.2.
 
 Project website https://osdn.net/projects/ssperl/
 
-=head1 DESCRIPTION
-
-Net::Shadowsocks is a Perl implementation of the shadowsocks (Chinese: 影梭) 
-protocol client and server , Shadowsocks is a secure transport protocol based on 
-SOCKS Protocol Version 5 (RFC 1928 ). Currently the only encryption method 
-supported is RC4-MD5 which is introduced into shadowsocks Python version 2.2. 
-
-Project website https://osdn.net/projects/ssperl/
-
 Net::Shadowsocks
 
 =head1 VERSION
 
-Version 0.6.0
+Version 0.6.1
 
 =head1 SUBROUTINES/METHODS
 
@@ -710,7 +750,7 @@ use the server module.
 
 =head1 AUTHOR
 
-Li ZHOU, C<< <zhou384 at gmail.com> >>
+Li ZHOU, C<< <lzh at cpan.org> >>
 
 =head1 BUGS
 
@@ -790,7 +830,7 @@ EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 =head1 AUTHOR
 
-Li ZHOU <zhou384@gmail.com>
+Li ZHOU <lzh@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
